@@ -13,23 +13,23 @@ import time
 from collections import deque
 from functools import total_ordering
 from heapq import heappush, heappop
+from threading import get_ident
 
-from hazelcast.config import SSLProtocol
+from hazelcast.config import SSLProtocol, _Config
 from hazelcast.connection import Connection
 from hazelcast.core import Address
 from hazelcast.errors import HazelcastError
 from hazelcast.future import Future
 
 try:
+    # Available in UNIX.
+    # It is used to make pipes used to wake the event
+    # loop non-blocking.
     import fcntl
-except ImportError:
-    fcntl = None
 
-try:
-    from _thread import get_ident
+    _FCNTL_EXISTS = True
 except ImportError:
-    # Python2
-    from thread import get_ident
+    _FCNTL_EXISTS = False
 
 _logger = logging.getLogger(__name__)
 
@@ -45,21 +45,26 @@ _logger = logging.getLogger(__name__)
 _RETRYABLE_ERROR_CODES = (
     errno.EAGAIN,
     errno.EWOULDBLOCK,
-    errno.EDEADLK,
+    # Missing in the stub, but present in the implementation.
+    # https://docs.python.org/3/library/errno.html#errno.EDEADLK
+    # Added to the stub in
+    # https://github.com/python/typeshed/pull/7397
+    # For now, we will ignore the mypy error.
+    errno.EDEADLK,  # type: ignore[attr-defined]
     ssl.SSL_ERROR_WANT_WRITE,
     ssl.SSL_ERROR_WANT_READ,
 )
 
 
 def _set_nonblocking(fd):
-    if not fcntl:
+    if not _FCNTL_EXISTS:
         return
 
     flags = fcntl.fcntl(fd, fcntl.F_GETFL)
     fcntl.fcntl(fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)
 
 
-class _SocketAdapter(object):
+class _SocketAdapter:
     def __init__(self, fd):
         self._fd = fd
 
@@ -158,7 +163,7 @@ class _SocketedWaker(_AbstractWaker):
         self._writer.close()
 
 
-class _AbstractLoop(object):
+class _AbstractLoop:
     def __init__(self, map):
         self._map = map
         self._timers = []  # Accessed only from the reactor thread
@@ -284,7 +289,7 @@ class _WakeableLoop(_AbstractLoop):
                 continue
 
             try:
-                connection.close(None, HazelcastError("Client is shutting down"))
+                connection.close_connection(None, HazelcastError("Client is shutting down"))
             except OSError as connection:
                 if connection.args[0] == socket.EBADF:
                     pass
@@ -316,7 +321,7 @@ class _BasicLoop(_AbstractLoop):
 
         for connection in list(self._map.values()):
             try:
-                connection.close(None, HazelcastError("Client is shutting down"))
+                connection.close_connection(None, HazelcastError("Client is shutting down"))
             except OSError as connection:
                 if connection.args[0] == socket.EBADF:
                     pass
@@ -326,7 +331,7 @@ class _BasicLoop(_AbstractLoop):
         self._map.clear()
 
 
-class AsyncoreReactor(object):
+class AsyncoreReactor:
     def __init__(self):
         self.map = {}
         loop = None
@@ -391,8 +396,9 @@ class AsyncoreConnection(Connection, asyncore.dispatcher):
         # set the socket timeout to 0 explicitly
         self.socket.settimeout(0)
         self._set_socket_options(config)
+
         if config.ssl_enabled:
-            self._wrap_as_ssl_socket(config)
+            self._wrap_as_ssl_socket(config, address.host)
 
         try:
             self.connect((address.host, address.port))
@@ -432,7 +438,7 @@ class AsyncoreConnection(Connection, asyncore.dispatcher):
         except socket.error as err:
             if err.args[0] not in _RETRYABLE_ERROR_CODES:
                 # Other error codes are fatal, should close the connection
-                self.close(None, err)
+                self.close_connection(None, err)
 
         if reader.length:
             reader.process()
@@ -473,7 +479,7 @@ class AsyncoreConnection(Connection, asyncore.dispatcher):
                 self._write_queue.appendleft(bytes_)
             else:
                 # Other error codes are fatal, should close the connection
-                self.close(None, err)
+                self.close_connection(None, err)
         else:
             # No exception is thrown during the send
             self.last_write_time = time.time()
@@ -484,14 +490,14 @@ class AsyncoreConnection(Connection, asyncore.dispatcher):
 
     def handle_close(self):
         _logger.warning("Connection closed by server")
-        self.close(None, IOError("Connection closed by server"))
+        self.close_connection(None, IOError("Connection closed by server"))
 
     def handle_error(self):
         # We handle retryable error codes inside the
         # handle_read/write. Anything else should be fatal.
         error = sys.exc_info()[1]
         _logger.debug("Received error", exc_info=True)
-        self.close(str(error), None)
+        self.close_connection(str(error), None)
 
     def readable(self):
         return self.live and self.sent_protocol_bytes
@@ -511,12 +517,12 @@ class AsyncoreConnection(Connection, asyncore.dispatcher):
             # no effects.
             self._close_timer.cancel()
 
-        asyncore.dispatcher.close(self)
+        self.close()
         self._write_buf.close()
 
     def _close_timer_cb(self):
         if not self.connected:
-            self.close(None, IOError("Connection timed out"))
+            self.close_connection(None, IOError("Connection timed out"))
 
     def _set_socket_options(self, config):
         # set tcp no delay
@@ -533,7 +539,7 @@ class AsyncoreConnection(Connection, asyncore.dispatcher):
 
             self.socket.setsockopt(level, option_name, value)
 
-    def _wrap_as_ssl_socket(self, config):
+    def _wrap_as_ssl_socket(self, config: _Config, hostname: str):
         ssl_context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
 
         protocol = config.ssl_protocol
@@ -570,7 +576,18 @@ class AsyncoreConnection(Connection, asyncore.dispatcher):
         if config.ssl_ciphers:
             ssl_context.set_ciphers(config.ssl_ciphers)
 
-        self.socket = ssl_context.wrap_socket(self.socket)
+        server_hostname = None
+        if config.ssl_check_hostname:
+            ssl_context.check_hostname = True
+            server_hostname = hostname
+
+        # self.socket is initialized with self.create_socket call, which
+        # made before this. So, mypy error regarding possible None
+        # value of the self.socket is not valid.
+        self.socket = ssl_context.wrap_socket(
+            self.socket,  # type: ignore[arg-type]
+            server_hostname=server_hostname,
+        )
 
     def __repr__(self):
         return "Connection(id=%s, live=%s, remote_address=%s)" % (
@@ -584,7 +601,7 @@ class AsyncoreConnection(Connection, asyncore.dispatcher):
 
 
 @total_ordering
-class Timer(object):
+class Timer:
     __slots__ = ("end", "timer_ended_cb", "canceled")
 
     def __init__(self, end, timer_ended_cb):
