@@ -1,18 +1,12 @@
-import binascii
 import random
 import threading
 import time
 import uuid
 
+from collections.abc import Sequence, Iterable
+
 from hazelcast.serialization import UUID_MSB_SHIFT, UUID_LSB_MASK, UUID_MSB_MASK
 
-try:
-    from collections.abc import Sequence, Iterable
-except ImportError:
-    # Compatibility for Python2
-    from collections import Sequence, Iterable
-
-from hazelcast import six
 
 DEFAULT_ADDRESS = "127.0.0.1"
 DEFAULT_PORT = 5701
@@ -47,7 +41,7 @@ def check_is_number(val, message="Number value expected"):
 
 
 def check_is_int(val, message="Int value expected"):
-    if not isinstance(val, six.integer_types):
+    if not isinstance(val, int):
         raise AssertionError(message)
 
 
@@ -83,37 +77,48 @@ def validate_serializer(serializer, _type):
         raise ValueError("Serializer should be an instance of %s" % _type.__name__)
 
 
-class AtomicInteger(object):
+class AtomicInteger:
     """An Integer which can work atomically."""
 
-    def __init__(self, initial=0):
+    def __init__(self, initial: int = 0):
         self._mux = threading.RLock()
         self._counter = initial
 
-    def get_and_increment(self):
+    def get_and_increment(self) -> int:
         """Returns the current value and increment it.
 
         Returns:
-            int: Current value of AtomicInteger.
+            Current value of AtomicInteger.
         """
         with self._mux:
             res = self._counter
             self._counter += 1
             return res
 
-    def get(self):
+    def increment_and_get(self) -> int:
+        """Increments the current value and returns it.
+
+        Returns:
+            Incremented value of AtomicInteger.
+        """
+        with self._mux:
+            self._counter += 1
+            return self._counter
+
+    def get(self) -> int:
         """Returns the current value.
 
         Returns:
-            int: The current value.
+            The current value.
         """
         with self._mux:
             return self._counter
 
-    def add(self, count):
+    def add(self, count: int):
         """Adds the given value to the current value.
+
         Args:
-            count (int): The value to add.
+            count: The value to add.
         """
         with self._mux:
             self._counter += count
@@ -215,14 +220,14 @@ def to_signed(unsigned, bit_len):
 
 
 def get_attr_name(cls, value):
-    for attr_name, attr_value in six.iteritems(vars(cls)):
+    for attr_name, attr_value in vars(cls).items():
         if attr_value == value:
             return attr_name
     return None
 
 
 def _get_enum_value(cls, key):
-    if isinstance(key, six.string_types):
+    if isinstance(key, str):
         return getattr(cls, key, None)
     return None
 
@@ -248,23 +253,24 @@ def try_to_get_enum_value(value, enum_class):
             )
 
 
-number_types = (six.integer_types, float)
-none_type = type(None)
+number_types = (int, float)
 
 
-class LoadBalancer(object):
-    """Load balancer allows you to send operations to one of a number of endpoints (Members).
-    It is up to the implementation to use different load balancing policies.
+class LoadBalancer:
+    """Load balancer allows you to send operations to one of a number of
+    endpoints (Members). It is up to the implementation to use different
+    load balancing policies.
 
-    If the client is configured with smart routing,
-    only the operations that are not key based will be routed to the endpoint
+    If the client is configured with smart routing, only the operations that
+    are not key based will be routed to the endpoint
     """
 
     def init(self, cluster_service):
         """Initializes the load balancer.
 
         Args:
-            cluster_service (hazelcast.cluster.ClusterService): The cluster service to select members from
+            cluster_service (hazelcast.cluster.ClusterService): The cluster
+                service to select members from
         """
         raise NotImplementedError("init")
 
@@ -272,80 +278,39 @@ class LoadBalancer(object):
         """Returns the next member to route to.
 
         Returns:
-            hazelcast.core.MemberInfo: the next member or ``None`` if no member is available.
+            hazelcast.core.MemberInfo: the next member or ``None`` if no member
+            is available.
         """
         raise NotImplementedError("next")
 
-    def next_data_member(self):
-        """Returns the next data member to route to.
-
-        Returns:
-            hazelcast.core.MemberInfo: The next data member or
-            ``None`` if no data member is available.
-        """
-        return None
-
-    def can_get_next_data_member(self):
-        """Returns whether this instance supports getting data members
-         through a call to :func:`next_data_member`.
-
-        Returns:
-            bool: ``True`` if this instance supports getting data members.
-        """
-        return False
-
-
-class _Members(object):
-    __slots__ = ("members", "data_members")
-
-    def __init__(self, members, data_members):
-        self.members = members
-        self.data_members = data_members
-
 
 class _AbstractLoadBalancer(LoadBalancer):
     def __init__(self):
         self._cluster_service = None
-        self._members = _Members([], [])
+        self._members = []
 
     def init(self, cluster_service):
         self._cluster_service = cluster_service
         cluster_service.add_listener(self._listener, self._listener, True)
 
-    def next(self):
-        members = self._members.members
-        return self._next(members)
-
-    def next_data_member(self):
-        members = self._members.data_members
-        return self._next(members)
-
-    def can_get_next_data_member(self):
-        return True
-
     def _listener(self, _):
-        members = self._cluster_service.get_members()
-        data_members = [member for member in members if not member.lite_member]
-
-        self._members = _Members(members, data_members)
-
-    def _next(self, members):
-        raise NotImplementedError("_next")
+        self._members = self._cluster_service.get_members()
 
 
 class RoundRobinLB(_AbstractLoadBalancer):
     """A load balancer implementation that relies on using round robin
     to a next member to send a request to.
 
-    Round robin is done based on best effort basis, the order of members for concurrent calls to
-    the next() is not guaranteed.
+    Round robin is done based on best effort basis, the order of members for
+    concurrent calls to the next() is not guaranteed.
     """
 
     def __init__(self):
         super(RoundRobinLB, self).__init__()
         self._idx = 0
 
-    def _next(self, members):
+    def next(self):
+        members = self._members
         if not members:
             return None
 
@@ -358,14 +323,15 @@ class RoundRobinLB(_AbstractLoadBalancer):
 class RandomLB(_AbstractLoadBalancer):
     """A load balancer that selects a random member to route to."""
 
-    def _next(self, members):
+    def next(self):
+        members = self._members
         if not members:
             return None
         idx = random.randrange(0, len(members))
         return members[idx]
 
 
-class IterationType(object):
+class IterationType:
     """To differentiate users selection on result collection on map-wide
     operations like ``entry_set``, ``key_set``, ``values`` etc.
     """
@@ -380,7 +346,7 @@ class IterationType(object):
     """Iterate over entries"""
 
 
-class UUIDUtil(object):
+class UUIDUtil:
     @staticmethod
     def to_bits(value):
         i = value.int
@@ -398,48 +364,20 @@ class UUIDUtil(object):
         )
 
 
-if hasattr(int, "from_bytes"):
-
-    def int_from_bytes(buffer):
-        return int.from_bytes(buffer, "big", signed=True)
+def int_from_bytes(buf):
+    return int.from_bytes(buf, "big", signed=True)
 
 
-else:
-    # Compatibility with Python 2
-    def int_from_bytes(buffer):
-        buffer = bytearray(buffer)
-        if buffer[0] & 0x80:
-            neg = bytearray()
-            for c in buffer:
-                neg.append(c ^ 0xFF)
-            return -1 * int(binascii.hexlify(neg), 16) - 1
-        return int(binascii.hexlify(buffer), 16)
-
-
-try:
-    from datetime import timezone
-except ImportError:
-    from datetime import tzinfo, timedelta
-
-    # There is no tzinfo implementation(timezone) in the
-    # Python 2. Here we provide the bare minimum
-    # to the user.
-    class FixedOffsetTimezone(tzinfo):
-        __slots__ = ("_offset",)
-
-        def __init__(self, offset):
-            self._offset = offset
-
-        def utcoffset(self, dt):
-            return self._offset
-
-        def tzname(self, dt):
-            return None
-
-        def dst(self, dt):
-            return timedelta(0)
-
-    timezone = FixedOffsetTimezone
+def int_to_bytes(number):
+    # Number of bytes to represent the number.
+    # For numbers that don't have exactly 8n bit_length,
+    # adding 8 and performing integer division with 8
+    # let us get the correct length because
+    # (8n + m + 8) // 8 = n + 0 + 1 (assuming m < 8).
+    # For negative numbers, we add 1 to get rid of the
+    # effects of the leading 1 (the sign bit).
+    width = (8 + (number + (number < 0)).bit_length()) // 8
+    return number.to_bytes(length=width, byteorder="big", signed=True)
 
 
 def try_to_get_error_message(error):
@@ -454,3 +392,78 @@ def try_to_get_error_message(error):
     elif len(error.args) > 0:
         return error.args[0]
     return None
+
+
+def _is_same_version(v1, v2):
+    # Ignores the patch version
+    return v1.major == v2.major and v1.minor == v2.minor
+
+
+def _is_newer_version(v1, v2):
+    # Ignores the patch version
+    return v1.major > v2.major or (v1.major == v2.major and v1.minor > v2.minor)
+
+
+def member_of_larger_same_version_group(members):
+    """Finds a larger same-version group of data members from a collection of
+    members and return a random member from the group.
+
+    If the same-version groups have the same size, return a member from the
+    newer group.
+
+    Args:
+        members (list[hazelcast.core.MemberInfo]): List of all members.
+
+    Returns:
+        hazelcast.core.MemberInfo: The chosen member or ``None``, if no data
+        member is found.
+    """
+    # The members should have at most 2 different version (ignoring the patch
+    # version).
+
+    version0 = None
+    version1 = None
+    count0 = 0
+    count1 = 0
+
+    for member in members:
+        if member.lite_member:
+            continue
+
+        v = member.version
+        if not version0 or _is_same_version(version0, v):
+            version0 = v
+            count0 += 1
+        elif not version1 or _is_same_version(version1, v):
+            version1 = v
+            count1 += 1
+        else:
+            raise ValueError(
+                "More than 2 distinct member versions found: %s, %s, %s" % (version0, version1, v)
+            )
+
+    # no data members
+    if count0 == 0:
+        return None
+
+    if count0 > count1 or count0 == count1 and _is_newer_version(version0, version1):
+        count = count0
+        version = version0
+    else:
+        count = count0
+        version = version1
+
+    # return a random member from the larger group
+    random_member_idx = random.randrange(0, count)
+    for member in members:
+        if not member.lite_member and _is_same_version(version, member.version):
+            random_member_idx -= 1
+            if random_member_idx < 0:
+                return member
+
+
+def re_raise(exception, traceback):
+    if exception.__traceback__ is not traceback:
+        raise exception.with_traceback(traceback)
+
+    raise exception
